/* TESTARGPARSER.C - Test harness for the argparser module.

            testarggparser.c provides the interface between testharness
            functions and the C functions being tested. It contains
            two functions:
            
            - show_help(), which provides user information about
                           the tests to be run, such as script names
                           and any external requirements, apart from
                           test scripts, needed to do the tests.
                           
            - calladapter(), which allows individual function tests to be
                           run by mapping the function name and arguments
                           in the test definition to a function call,
                           executing the function, and writing test results
                           to stdout after the test has been run.

            Tests are driven by testharness.c, which uses command line
            arguments and options to set run conditions and specify
            which test scripts will be run. Testharness interprets each
            test script, using the calladapter() function in this module
            to exercise the functions being tested, and writes the results
            of each call to stdout.

            Written by M.C.Gregorie.

            Version history
               
            1.0   28/06/20   Initial release
            2.0   31/07/20   Renamed testharness() to runtest()
*/

#define VERSION         "2.0"
#define COPYRIGHT       "(c) M.C.Gregorie, 2020"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "../argparser.h"
#include "runtest.h"

/*
   Static variables used to define commandname, argc and argv[]
*/
char  progid[MAXLINE+1];
int   argc = 0;
char  *argv[20];

/*
   Private function prototypes
*/
int  example(char *s, int n);
void testargs(char s[MAXARGS][MAXLINE], int n[MAXARGS]);
void testend(int reply, char *fn, char s[MAXARGS][MAXLINE], int n[MAXARGS]);
void teststart(char *fn, char s[MAXARGS][MAXLINE], int n[MAXARGS]);

/*
   A code-specific call adapter which must be modified to call the
   set of functions to be tested.

   Entry and exit is reported if debug level is 2 or more.
   This example implementation contains the code to run one example
   function, whose prototype is:

   int example(char *s int i);

   to show a basic setup. The basic if..then..else chain of function calls
   should be preserved, but any or all if-actions can be converted into code
   blocks and/or may call additional privately defined functions.

   The functions used to return results, report errors, etc, are defined
   in testharness.c as the setxxxxreply() functions.

   The final item in the condition chain should not be modified. It is there to
   trap function names in test scripts that don't match any of the functions
   being tested.

   When call adapter is called by testharness.c:

   - debug (normally zero) may be set to 1 to report calladapter entry and
           exit conditions.

   - fn    must contains the name of the function being tested

   - s[][] is an array containing the call parameters as strings

   - n[]   is an array containing the call parameters as int values with
           those that weren't numeric set to zero.

   It is the customiser's responsibility to map the contents of the
   s[][] and n[] arrays to the parameter list of each function being
   tested. Any additional code that may is needed to provide the correct
   mapping can be added as required.

   On exit calladapter() should return 0 if the overall call didn't
   encounter any fatal errors and a non-zero value if an error was
   encountered that was serious enough to abandon the run part-way
   through the test script.
*/
int calladapter(int debug, char *fn, char s[MAXARGS][MAXLINE], int n[MAXARGS])
{
   int   i;
   int   reply = 0;
   char  emsg[MAXLINE+1];
   char  *name;
   char  *value;
   
   if (debug)
      setargparserdebug();

   if (debug > 1)
      teststart(fn, s, n);

   if (strcmp(fn, "exec") == 0)
   {
      /*
         The exec line is written as:

         exec progname args...
      */
      argc = 0;
      for (i = 0; i < 20; i++)
      {
         if (strlen(s[i]) > 0)
         {
            if (i == 0)
               strcpy(progid, s[i]);

            argv[i] = malloc(strlen(s[i]) + 1);   
            strcpy(argv[i], s[i]);
            argc++;
         }
         else
            break;
      }

      printf("Exec: ");
      for (i = 0; i < argc; i++)
         printf("%s ", argv[i]);

      printf("\n");
      printf("      progname : %s\n", progid);
      printf("      arguments: [", argc);
      for (i = 0; i < argc; i++)
      {
         if (i > 0)
            printf(",");

         printf(argv[i]);
      }

      printf("]\n\n"); 
   }
   else
   if (strcmp(fn, "outputs") == 0)
   {
      printf("Outputs\n");
      printf("   Options:\n");
      i = 0;
      while(TRUE)
      {
         name = nextoption();
         if (name == NULL)
            break;
            
         value = getvalue();         
         printf("      %2d %s %s %s\n",
                i++,
                name,
                ( value == NULL ? "" : "->"),
                ( value == NULL ? "" : value));
      }
      
      printf("   Arguments:\n");
      i = 0;
      while(TRUE)
      {
         value = nextargument();
         if (value == NULL)
             break;
             
         printf("      %2d %s\n", i++, value);
      }

      printf("\n");
   }
   else
   if (strcmp(fn, "argparser") == 0)
   {
      /* test is written:

         argparser optlist

         with the other arguments derived from the 'exec' line
      */
      argparser(argc, argv, s[0]);
   }
   else
   if (strcmp(fn, "parse") == 0)
   {
      parse();
   }
   else
   if (strcmp(fn, "getprogname") == 0)
   {
      setstringreply(getprogname());
   }
   else
   if (strcmp(fn, "getvalue") == 0)
   {
      setstringreply(getvalue());
   }
   else
   if (strcmp(fn, "nextargument") == 0)
   {
      setstringreply(nextargument());
   }
   else
   if (strcmp(fn, "nextoption") == 0)
   {
      setstringreply(nextoption());
   }
   else
   if (strcmp(fn, "setcasesensitive") == 0)
   {
      setcasesensitive(n[0]);
   }
   else
   if (strcmp(fn, "setargparserdebug") == 0)
   {
      setargparserdebug();
   }
   else
   {
      snprintf(emsg, MAXLINE, "unknown function: %s", fn);
      seterror(emsg);
   }

   if (debug > 1)
      testend(reply, fn, s, n);

   return reply;
}
 
/*
   The text in showhelp() is displayed if the test harness is run
   with the '-?' option. It should say that it is a customised
   test harness, describe what functions or module are being tested
   and identify the (set of) test scripts it uses. 
*/
void showhelp()
{
   printf("\nargparser v%s %s\n\n",VERSION,COPYRIGHT);
   printf("Syntax:   argparser [<opts>] <testfile...> [<opts>]\n");
   printf("Function: Argparser test harness.\n\n");
   printf("          Test scripts are called data/ap.99txt\n\n");
   printf("          NOTE 1: a pseudo test line starting with 'exec' must\n");
   printf("          preceed the 'argparser' test line. It defines the\n");
   printf("          program name, command line  arguments and options to\n");
   printf("          be used as test data. An example is:\n\n");
   printf("          exec test01 -dd textval.txt 23\n\n");
   printf("          NOTE 2: pseudo test 'outputs' lists all opts and args\n");
   printf("          extracted from the commandline.\n");
   printf("Options:\n");
   printf("     -d or -dd shows 'exec' results and enables argparser\n");
   printf("               internal debugging using 'setargparserdebug()\n");
   printf("     -d -d     a second '-d' also shows the function name and\n");
   printf("               arguments before  running each function test\n");
}

/*
   Private functions start here
   ============================
*/

/*
   example() is a dummy function, put here so that the prototype
   custom.c source file will compile, link and run the custom01.txt
   test script.
*/
int example(char *s, int n)
{
   printf("%s\n", s);
   return n;
}

/*
   testargs() is a debug aid that lists the supplied function parameters
*/
void testargs(char s[MAXARGS][MAXLINE], int n[MAXARGS])
{
   int i;

   fprintf(stderr, "    ");
   for (i = 0; i < MAXARGS && strlen(s[i]) > 0; i++)
      fprintf(stderr, "s[%d]='%s' ", i, s[i]);

   fprintf(stderr, "\n    ");
   for (i = 0; i < MAXARGS && strlen(s[i]) > 0; i++)
      fprintf(stderr, "n[%d]=%d ", i, n[i]);

   fprintf(stderr, "\n"); 
}

/*
   testend() is a debug aid called on exit from calladapter()
*/
void testend(int reply, char *fn, char s[MAXARGS][MAXLINE], int n[MAXARGS])
{
   fprintf(stderr, "%d = calladapter exit after %s\n", reply, fn);
   testargs(s, n);
}

/*
   teststart() is a debug aid called on entry to calladapter()
*/
void teststart(char *fn, char s[MAXARGS][MAXLINE], int n[MAXARGS])
{
   fprintf(stderr, "calladapter entered for %s\n", fn);
   testargs(s, n);
}
