/* TESTRE.C - Test harness for the reporterror diagnostic module.

            testre.c provides the reporterror-specific functions needed
            to exercise reporterror function calls and report results.
            These functions are:
            - show_help(), which provides user information about
                           reporterror regression tests
            - calladapter(), which runs individual function tests
                           by mapping the function and arguments in
                           the line defining each function test
                           into a function call and writing test results
                           to stdout immediately after the line
                           triggering the test.

            Tests are driven by testharness, which uses command line
            arguments and options to set run conditions and specify
            which test scripts will be run. Testharness interprets each
            test script, using the calladapter() function in this module
            to exercise reporterror functions and writing the results
            of each call to stdout.

            Written by M.C.Gregorie.

            Version history
               
            1.0   26/05/20   Initial release
            2.0   31/07/20   Renamed testharness() to runtest()
*/

#define VERSION         "2.0"
#define COPYRIGHT       "(c) M.C.Gregorie, 2020"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "../reporterror.h"
#include "runtest.h"
#include "testre.h"

/*
   Private function prototypes
*/
void testargs(char s[MAXARGS][MAXLINE], int n[MAXARGS]);
void testend(int reply, char *fn, char s[MAXARGS][MAXLINE], int n[MAXARGS]);
void teststart(char *fn, char s[MAXARGS][MAXLINE], int n[MAXARGS]);

/*
   This calladapter() instance has been customised to test all the 
   functions in the reporterror diagnostic module.
   Entry and exit is reported if debug level is 2 or more
*/
int calladapter(int debug, char *fn, char s[MAXARGS][MAXLINE], int n[MAXARGS])
{
   int   reply = 0;
   char  emsg[MAXLINE+1];

   if (debug > 1)
      teststart(fn, s, n);

   if (strcmp(fn, "reporterror") == 0)
      reporterror(s[0]);
   else
   if (strcmp(fn, "re_error") == 0)
      re_error(s[0],s[1],s[2],s[3],s[4],s[5],s[6],s[7],s[8],s[9]);
   else
   if (strcmp(fn, "re_getdebug") == 0)
      setintreply(re_getdebug());
   else
   if (strcmp(fn, "re_inrange") == 0)
      setboolreply(re_inrange(n[0], n[1]));
   else
   if (strcmp(fn, "re_istraceable") == 0)
      setboolreply(re_istraceable(n[0]));
   else
   if (strcmp(fn, "re_ltrace") == 0)
      re_ltrace(n[0],s[1],s[2],s[3],s[4],s[5],s[6],s[7],s[8],s[9]);
   else
   if (strcmp(fn, "re_println") == 0)
      re_println(s[0],s[1],s[2],s[3],s[4],s[5],s[6],s[7],s[8],s[9]);
   else
   if (strcmp(fn, "re_setbuffersize") == 0)
      re_setbuffersize(n[0]);
   else
   if (strcmp(fn, "re_setdebug") == 0)
      re_setdebug(n[0]);
   else
   if (strcmp(fn, "re_trace") == 0)
      re_trace(s[0],s[1],s[2],s[3],s[4],s[5],s[6],s[7],s[8],s[9]);
   else
   if (strcmp(fn, "re_usebuffer") == 0)
      re_usebuffer(n[0]);
   else
   if (strcmp(fn, "re_warn") == 0)
      re_warn(s[0],s[1],s[2],s[3],s[4],s[5],s[6],s[7],s[8],s[9]);
   else
   {
      snprintf(emsg, MAXLINE, "unknown function: %s", fn);
      seterror(emsg);
   }

   if (debug > 1)
      testend(reply, fn, s, n);

   return reply;
}
 
/*
   Summarises the functions this module tests and shows what test scripts
   it uses.
*/
void showhelp()
{
   printf("\nreporterror v%s %s\n\n",VERSION,COPYRIGHT);
   printf("Syntax  : reporterror [<opts>] <testfile...> [<opts>]\n");
   printf("Function: Test harness for the reporterror diagnostic module.\n");
   printf("          Test scripts are called data/re99.txt\n");
   printf("Options :\n");
   printf("     -d           debug mode - repeat for more detail\n");
}

/*
   testargs() is a debug aid that lists the supplied function parameters
*/
void testargs(char s[MAXARGS][MAXLINE], int n[MAXARGS])
{
   int i;

   fprintf(stderr, "    ");
   for (i = 0; i < MAXARGS && strlen(s[i]) > 0; i++)
      fprintf(stderr, "s[%d]='%s' ", i, s[i]);

   fprintf(stderr, "\n    ");
   for (i = 0; i < MAXARGS && strlen(s[i]) > 0; i++)
      fprintf(stderr, "n[%d]=%d ", i, n[i]);

   fprintf(stderr, "\n"); 
}

/*
   testend() is a debug aid called on exit from calladapter()
*/
void testend(int reply, char *fn, char s[MAXARGS][MAXLINE], int n[MAXARGS])
{
   fprintf(stderr, "%d = calladapter exit after %s\n", reply, fn);
   testargs(s, n);
}

/*
   teststart() is a debug aid called on entry to calladapter()
*/
void teststart(char *fn, char s[MAXARGS][MAXLINE], int n[MAXARGS])
{
   fprintf(stderr, "calladapter entered for %s\n", fn);
   testargs(s, n);
}
